
/* --------------------------------------------------------------------  */
/*                                                                       */
/*                         Author: Dave Shield                           */
/*                        Computer Sceince Dept                          */
/*                   University of Liverpool, England                    */
/*                                                                       */
/*                       D.T.Shield@csc.liv.ac.uk                        */
/*                                                                       */
/* --------------------------------------------------------------------  */
/*
 *	pwck_hpux - Perform HP-UX password checks.
 *
 *		at least two alphabetic characters
 *		at least one numeric or special character
 *		password does not equal the username
 *		  (forwards or reversed), or any circular shift
 *
 *	Returns:
 *		PWCK_OBVIOUS if <password> fails any of the above
 *		PWCK_FAIL if failure to get the username
 *		PWCK_OK if otherwise
 */

#include "checkpasswd.h"
#include <pwd.h>

pwck_hpux(password, userid, mesg)
char	*password;	/* Password to check */
int	userid;		/* Calling userid */
char	*mesg;		/* Message buffer */
{
	char	temp[BUFSIZ];		/* To store username rotations */
	int	namelen;		/* Length of username */
	struct passwd	*pwp;		/* Pointer to user information */
	char	*p;

	int	alpha = 0,		/* Count of alphabetics */
		num_sp = 0;		/* Count of numbers/specials */

#ifdef  DEBUG
        printf("pwck_hpux: \"%s\"\n", password);
#endif


		/*
		 *  Check for shifted username
		 *
		 *  Double the username, and check fixed-length
		 *	substrings, to cover all rotations.
		 */
	if (userid < 0)			/* Can't do user checks*/
		return(PWCK_FAIL);
	if ((pwp = getpwuid(userid)) == (struct passwd *)0)
		 return(PWCK_FAIL);

	(void)strcpy( temp, pwp->pw_name );
	(void)strcat( temp, pwp->pw_name );
	namelen = strlen( pwp->pw_name );
	strcpy(mesg, "Password is based on your username");

	for (p = &(temp[namelen]) ; p > temp ; ) {
		try(password, p, PWCK_OBVIOUS);
		p--;
		*(p+namelen) = '\0';
	}

		/*
		 *  Check for shifted reversed username
		 */
	(void)strcpy( temp, pwp->pw_name );
	(void)strcat( temp, pwp->pw_name );
	_flipstring(temp, 0);

	for (p = &(temp[namelen]) ; p > temp ; ) {
		try(password, p, PWCK_OBVIOUS);
		p--;
		*(p+namelen) = '\0';
	}

		/*
		 *  Check for shifted reversed username
		 */
	for ( p=password ; *p ; p++ ) {
		if (isalpha(*p))
			alpha++;
		else if (isdigit(*p) || ispunct(*p) || isspace(*p))
			num_sp++;
	}
	if (( alpha < 2 ) || ( num_sp < 1 )) {
		(void) strcpy( mesg,
"Password must contain at least two alphabetic characters,\n\
and at least one digit or special character.");
		return(PWCK_OBVIOUS);
	}

	return(PWCK_OK);
}
/*	End pwck_hpux.c */
